import random

import requests
from netaddr import IPNetwork


def generate_test_addresses():
    prefixes = requests.get('https://ip-ranges.amazonaws.com/ip-ranges.json').json()['prefixes']
    nets = [IPNetwork(_['ip_prefix']) for _ in prefixes if _['region'].startswith('ap-northeast-')]
    addr = [random.choice(random.choice(nets)) for _ in range(200)]
    for a in addr:
        print(a)



import numpy as np
from sklearn.cluster import KMeans, AgglomerativeClustering
from netaddr import IPAddress, IPNetwork


def agglomerative(ipaddr: [IPAddress], max_n_clusters):
    hie = AgglomerativeClustering(n_clusters=max_n_clusters).fit(np.array([[int(_)] for _ in ipaddr]))
    cidrs = []
    for c in range(max_n_clusters):
        members = np.array(ipaddr)[hie.labels_ == c]
        xor = int(min(members)) ^ int(max(members))
        xor = f"{xor:032b}"
        plen = 32 if xor.find('1') == -1 else xor.find('1')
        cidrs.append(IPNetwork(f"{max(members)}/{plen}").cidr)
    cidrs = sorted(cidrs)

    aggr = []
    for ip in ipaddr:
        a = [_ for _ in cidrs if ip in _]
        assert len(a) >= 1
        aggr.append(a[0])
    aggr = sorted(list(set(aggr)))

    return aggr


def kmeans(ipaddr: [IPAddress], max_n_clusters):
    hie = KMeans(n_clusters=max_n_clusters).fit(np.array([[int(_)] for _ in ipaddr]))
    cidrs = []
    for c in range(max_n_clusters):
        members = np.array(ipaddr)[hie.labels_ == c]
        xor = int(min(members)) ^ int(max(members))
        xor = f"{xor:032b}"
        plen = 32 if xor.find('1') == -1 else xor.find('1')
        cidrs.append(IPNetwork(f"{max(members)}/{plen}").cidr)
    cidrs = sorted(cidrs)

    aggr = []
    for ip in ipaddr:
        a = [_ for _ in cidrs if ip in _]
        assert len(a) >= 1
        aggr.append(a[0])
    aggr = sorted(list(set(aggr)))

    return aggr



from hilbertcurve.hilbertcurve import HilbertCurve
import numpy as np


def hilbert_curve(p=12, n=2):
    hilbert_curve = HilbertCurve(p, n)
    coords = []
    i = 0
    while True:
        try:
            coords.append(hilbert_curve.coordinates_from_distance(i))
            i += 1
        except:
            break
    np.savez_compressed(f'hilvert_p{p}_n{n}.npz', np.array(coords))



from pathlib import Path
from setuptools import setup, find_packages

setup(
    name='ClustIPy',
    version='0.1.0',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/shuichiro-makigaki/ClustIPy',
    license='MIT',
    author='Shuichiro MAKIGAKI',
    author_email='shuichiro.makigaki@gmail.com',
    description='Clustering IPs',
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    entry_points='''
        [console_scripts]
        clustipy=clustipy.cmd.clustipy:main
    ''',
)



from netaddr import IPAddress
import click

import clustipy


@click.command()
@click.argument('infile', type=click.File())
@click.option('--cluster', help='Max number of clusters', required=True, type=int)
@click.option('--algorithm', help='Clustering algorithm (default is `agglomerative`)',
              type=click.Choice(['agglomerative', 'kmeans']), default='agglomerative')
def main(infile, cluster, algorithm):
    ipaddr = [IPAddress(_) for _ in infile.readlines()]
    result = None
    if algorithm == 'agglomerative':
        result = clustipy.agglomerative(ipaddr, cluster)
    elif algorithm == 'kmeans':
        result = clustipy.kmeans(ipaddr, cluster)

    for r in result:
        print(r)


if __name__ == '__main__':
    main()



from pathlib import Path

from unittest import TestCase
from netaddr import IPAddress

import clustipy


class TestClustIPy(TestCase):
    def setUp(self):
        self.ipaddr = [IPAddress(_) for _ in Path('tests/test_address').read_text().splitlines()]

    def test_agglomerative(self):
        result = clustipy.agglomerative(self.ipaddr, 20)
        self.assertLessEqual(len(result), 20)
        for a in self.ipaddr:
            self.assertGreater(len([_ for _ in result if a in _]), 0)

    def test_kmeans(self):
        result = clustipy.kmeans(self.ipaddr, 20)
        self.assertLessEqual(len(result), 20)
        for a in self.ipaddr:
            self.assertGreater(len([_ for _ in result if a in _]), 0)
